bl_info = {
    "name": "Separate Loose Plus",
    "author": "Your Name",
    "version": (1, 0),
    "blender": (3, 0, 0),
    "location": "View3D > Edit Mode > Mesh > Separate Loose Plus",
    "description": "Tools for separating and merging mesh objects with tracking",
    "warning": "",
    "doc_url": "",
    "category": "Mesh",
}

import bpy

# Dictionary to store relationships between separated objects
# Format: {child_object_name: parent_object_name}
separation_history = {}

class MESH_OT_separate_and_edit(bpy.types.Operator):
    """Separate selected geometry and switch to editing the new object"""
    bl_idname = "mesh.separate_and_edit"
    bl_label = "Separate and Edit"
    bl_options = {'REGISTER', 'UNDO'}
    
    @classmethod
    def poll(cls, context):
        return (context.active_object is not None and
                context.active_object.type == 'MESH' and
                context.mode == 'EDIT_MESH')
    
    def execute(self, context):
        # Store the original object (parent)
        parent_object = context.active_object
        parent_name = parent_object.name
        
        # Get objects before separation
        before = set(context.scene.objects)
        
        # Separate selected mesh
        bpy.ops.mesh.separate(type='SELECTED')
        
        # Get objects after separation
        after = set(context.scene.objects)
        
        # Find new object(s)
        new_objects = list(after - before)
        if new_objects:
            # Switch to object mode
            bpy.ops.object.mode_set(mode='OBJECT')
            
            # Deselect all objects
            for obj in context.selected_objects:
                obj.select_set(False)
            
            # Select and make new object active
            new_object = new_objects[0]
            new_object.select_set(True)
            context.view_layer.objects.active = new_object
            
            # Store the relationship in our dictionary
            separation_history[new_object.name] = parent_name
            
            # Switch to edit mode
            bpy.ops.object.mode_set(mode='EDIT')
        
        return {'FINISHED'}


class MESH_OT_merge_back(bpy.types.Operator):
    """Merge this object back to its original object"""
    bl_idname = "mesh.merge_back"
    bl_label = "Merge Back to Object"
    bl_options = {'REGISTER', 'UNDO'}
    
    @classmethod
    def poll(cls, context):
        # Only enable if current object is in our separation history
        if context.active_object is None:
            return False
        return context.active_object.name in separation_history
    
    def execute(self, context):
        # Get the current object (child)
        child_obj = context.active_object
        child_name = child_obj.name
        
        # Get the parent object name from our dictionary
        parent_name = separation_history[child_name]
        
        # Check if parent still exists
        if parent_name not in bpy.data.objects:
            self.report({'ERROR'}, f"Original object '{parent_name}' no longer exists")
            return {'CANCELLED'}
        
        parent_obj = bpy.data.objects[parent_name]
        
        # Switch to object mode
        bpy.ops.object.mode_set(mode='OBJECT')
        
        # Select both objects, with parent as active
        bpy.ops.object.select_all(action='DESELECT')
        child_obj.select_set(True)
        parent_obj.select_set(True)
        context.view_layer.objects.active = parent_obj
        
        # Join the objects
        bpy.ops.object.join()
        
        # Remove from our dictionary since it's been merged
        if child_name in separation_history:
            del separation_history[child_name]
        
        # Stay in object mode
        
        return {'FINISHED'}


# Define the submenu
class VIEW3D_MT_separate_loose_plus(bpy.types.Menu):
    bl_label = "Separate Loose Plus"
    bl_idname = "VIEW3D_MT_separate_loose_plus"
    
    def draw(self, context):
        layout = self.layout
        
        # Add the separate and edit operator
        layout.operator(MESH_OT_separate_and_edit.bl_idname)
        
        # Only show merge back if it's applicable
        if context.active_object and context.active_object.name in separation_history:
            layout.operator(MESH_OT_merge_back.bl_idname)


# Function to add our submenu to the mesh menu
def add_submenu_to_mesh_menu(self, context):
    self.layout.separator()
    self.layout.menu(VIEW3D_MT_separate_loose_plus.bl_idname)


def register():
    print("Registering Separate Loose Plus - version without keymaps")
    bpy.utils.register_class(MESH_OT_separate_and_edit)
    bpy.utils.register_class(MESH_OT_merge_back)
    bpy.utils.register_class(VIEW3D_MT_separate_loose_plus)
    bpy.types.VIEW3D_MT_edit_mesh.append(add_submenu_to_mesh_menu)


def unregister():
    print("Unregistering Separate Loose Plus")
    bpy.types.VIEW3D_MT_edit_mesh.remove(add_submenu_to_mesh_menu)
    bpy.utils.unregister_class(VIEW3D_MT_separate_loose_plus)
    bpy.utils.unregister_class(MESH_OT_merge_back)
    bpy.utils.unregister_class(MESH_OT_separate_and_edit)


if __name__ == "__main__":
    register()
